# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...types.validation_requirement import ValidationRequirement
from ...types.expression_processing_response import ExpressionProcessingResponse
from ...core.request_options import RequestOptions
from ...types.expression_validation_result import ExpressionValidationResult
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.pydantic_utilities import parse_obj_as
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ExpressionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def validate(
        self,
        *,
        variables: typing.Sequence[ValidationRequirement],
        paper_expressions: ExpressionProcessingResponse,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpressionValidationResult:
        """
        Validates a set of variables against stored expressions to check for inconsistencies.
        Returns validation results for each relevant expression.

        Parameters
        ----------
        variables : typing.Sequence[ValidationRequirement]

        paper_expressions : ExpressionProcessingResponse

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpressionValidationResult
            Successful Response

        Examples
        --------
        from axiomatic import (
            Axiomatic,
            Expression,
            ExpressionProcessingResponse,
            Symbol,
            ValidationRequirement,
            ValidationResult,
        )

        client = Axiomatic(
            api_key="YOUR_API_KEY",
        )
        client.document.expression.validate(
            variables=[
                ValidationRequirement(
                    symbol=Symbol(
                        name="name",
                        wolfram_format="wolfram_format",
                        latex_representation="latex_representation",
                        dimension="dimension",
                        description="description",
                        type="scalar",
                        validations={
                            "key": ValidationResult(
                                is_valid=True,
                                message="message",
                            )
                        },
                    ),
                    value=1.1,
                )
            ],
            paper_expressions=ExpressionProcessingResponse(
                expressions=[
                    Expression(
                        name="name",
                        description="description",
                        original_format="original_format",
                        wolfram_expression="wolfram_expression",
                        symbols={
                            "key": Symbol(
                                name="name",
                                wolfram_format="wolfram_format",
                                latex_representation="latex_representation",
                                dimension="dimension",
                                description="description",
                                type="scalar",
                                validations={
                                    "key": ValidationResult(
                                        is_valid=True,
                                        message="message",
                                    )
                                },
                            )
                        },
                        narrative_assumptions=["narrative_assumptions"],
                        exp_validations={
                            "key": ValidationResult(
                                is_valid=True,
                                message="message",
                            )
                        },
                    )
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "document/expression/validate",
            method="POST",
            json={
                "variables": convert_and_respect_annotation_metadata(
                    object_=variables, annotation=typing.Sequence[ValidationRequirement], direction="write"
                ),
                "paper_expressions": convert_and_respect_annotation_metadata(
                    object_=paper_expressions, annotation=ExpressionProcessingResponse, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExpressionValidationResult,
                    parse_obj_as(
                        type_=ExpressionValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def process_depricated_version(
        self,
        *,
        markdown: str,
        images: typing.Optional[typing.Dict[str, str]] = OMIT,
        interline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        inline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpressionProcessingResponse:
        """
        Process all expressions at once and return their annotation

        Parameters
        ----------
        markdown : str

        images : typing.Optional[typing.Dict[str, str]]

        interline_equations : typing.Optional[typing.Sequence[str]]

        inline_equations : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpressionProcessingResponse
            Successful Response

        Examples
        --------
        from axiomatic import Axiomatic

        client = Axiomatic(
            api_key="YOUR_API_KEY",
        )
        client.document.expression.process_depricated_version(
            markdown="markdown",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "document/expression/process-depricated-version",
            method="POST",
            json={
                "markdown": markdown,
                "images": images,
                "interline_equations": interline_equations,
                "inline_equations": inline_equations,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExpressionProcessingResponse,
                    parse_obj_as(
                        type_=ExpressionProcessingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def process(
        self,
        *,
        markdown: str,
        images: typing.Optional[typing.Dict[str, str]] = OMIT,
        interline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        inline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpressionProcessingResponse:
        """
        process expressions from document in parallel

        Parameters
        ----------
        markdown : str

        images : typing.Optional[typing.Dict[str, str]]

        interline_equations : typing.Optional[typing.Sequence[str]]

        inline_equations : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpressionProcessingResponse
            Successful Response

        Examples
        --------
        from axiomatic import Axiomatic

        client = Axiomatic(
            api_key="YOUR_API_KEY",
        )
        client.document.expression.process(
            markdown="markdown",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "document/expression/process",
            method="POST",
            json={
                "markdown": markdown,
                "images": images,
                "interline_equations": interline_equations,
                "inline_equations": inline_equations,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExpressionProcessingResponse,
                    parse_obj_as(
                        type_=ExpressionProcessingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncExpressionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def validate(
        self,
        *,
        variables: typing.Sequence[ValidationRequirement],
        paper_expressions: ExpressionProcessingResponse,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpressionValidationResult:
        """
        Validates a set of variables against stored expressions to check for inconsistencies.
        Returns validation results for each relevant expression.

        Parameters
        ----------
        variables : typing.Sequence[ValidationRequirement]

        paper_expressions : ExpressionProcessingResponse

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpressionValidationResult
            Successful Response

        Examples
        --------
        import asyncio

        from axiomatic import (
            AsyncAxiomatic,
            Expression,
            ExpressionProcessingResponse,
            Symbol,
            ValidationRequirement,
            ValidationResult,
        )

        client = AsyncAxiomatic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.document.expression.validate(
                variables=[
                    ValidationRequirement(
                        symbol=Symbol(
                            name="name",
                            wolfram_format="wolfram_format",
                            latex_representation="latex_representation",
                            dimension="dimension",
                            description="description",
                            type="scalar",
                            validations={
                                "key": ValidationResult(
                                    is_valid=True,
                                    message="message",
                                )
                            },
                        ),
                        value=1.1,
                    )
                ],
                paper_expressions=ExpressionProcessingResponse(
                    expressions=[
                        Expression(
                            name="name",
                            description="description",
                            original_format="original_format",
                            wolfram_expression="wolfram_expression",
                            symbols={
                                "key": Symbol(
                                    name="name",
                                    wolfram_format="wolfram_format",
                                    latex_representation="latex_representation",
                                    dimension="dimension",
                                    description="description",
                                    type="scalar",
                                    validations={
                                        "key": ValidationResult(
                                            is_valid=True,
                                            message="message",
                                        )
                                    },
                                )
                            },
                            narrative_assumptions=["narrative_assumptions"],
                            exp_validations={
                                "key": ValidationResult(
                                    is_valid=True,
                                    message="message",
                                )
                            },
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "document/expression/validate",
            method="POST",
            json={
                "variables": convert_and_respect_annotation_metadata(
                    object_=variables, annotation=typing.Sequence[ValidationRequirement], direction="write"
                ),
                "paper_expressions": convert_and_respect_annotation_metadata(
                    object_=paper_expressions, annotation=ExpressionProcessingResponse, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExpressionValidationResult,
                    parse_obj_as(
                        type_=ExpressionValidationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def process_depricated_version(
        self,
        *,
        markdown: str,
        images: typing.Optional[typing.Dict[str, str]] = OMIT,
        interline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        inline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpressionProcessingResponse:
        """
        Process all expressions at once and return their annotation

        Parameters
        ----------
        markdown : str

        images : typing.Optional[typing.Dict[str, str]]

        interline_equations : typing.Optional[typing.Sequence[str]]

        inline_equations : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpressionProcessingResponse
            Successful Response

        Examples
        --------
        import asyncio

        from axiomatic import AsyncAxiomatic

        client = AsyncAxiomatic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.document.expression.process_depricated_version(
                markdown="markdown",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "document/expression/process-depricated-version",
            method="POST",
            json={
                "markdown": markdown,
                "images": images,
                "interline_equations": interline_equations,
                "inline_equations": inline_equations,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExpressionProcessingResponse,
                    parse_obj_as(
                        type_=ExpressionProcessingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def process(
        self,
        *,
        markdown: str,
        images: typing.Optional[typing.Dict[str, str]] = OMIT,
        interline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        inline_equations: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpressionProcessingResponse:
        """
        process expressions from document in parallel

        Parameters
        ----------
        markdown : str

        images : typing.Optional[typing.Dict[str, str]]

        interline_equations : typing.Optional[typing.Sequence[str]]

        inline_equations : typing.Optional[typing.Sequence[str]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpressionProcessingResponse
            Successful Response

        Examples
        --------
        import asyncio

        from axiomatic import AsyncAxiomatic

        client = AsyncAxiomatic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.document.expression.process(
                markdown="markdown",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "document/expression/process",
            method="POST",
            json={
                "markdown": markdown,
                "images": images,
                "interline_equations": interline_equations,
                "inline_equations": inline_equations,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ExpressionProcessingResponse,
                    parse_obj_as(
                        type_=ExpressionProcessingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .extracted_series_dto import ExtractedSeriesDto
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class PlotParserOutputV2Dto(UniversalBaseModel):
    """
    New plot parser output for v2, will replace old PlotParserOutput
    """

    extracted_series: typing.List[ExtractedSeriesDto]
    origin: typing.Optional[typing.List[typing.Optional[typing.Any]]] = None
    x_axis_len: typing.Optional[float] = None
    y_axis_len: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from .type import Type
import typing
from .validation_result import ValidationResult
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Symbol(UniversalBaseModel):
    """
    Represents a mathematical or physical symbol with its properties.

    Examples:
        >>> symbol = Symbol(
        ...     name="velocity",
        ...     wolfram_format="v",
        ...     latex_representation="\vec{v}",
        ...     dimension="meters/second",
        ...     description="Velocity of an object",
        ...     type="vector",
        ...     validations={},
        ... )

        >>> symbol = Symbol(
        ...     name="acceleration",
        ...     wolfram_format="a",
        ...     latex_representation="\vec{a}",
        ...     dimension="length * second^-2",
        ...     description="Rate of change of velocity",
        ...     type="vector",
        ...     validations={},
        ... )
    """

    name: str = pydantic.Field()
    """
    Name of the symbol, e.g., 'velocity', 'acceleration'
    """

    wolfram_format: str = pydantic.Field()
    """
    Wolfram Alpha format of the symbol
    """

    latex_representation: str = pydantic.Field()
    """
    LaTeX representation of the symbol
    """

    dimension: str = pydantic.Field()
    """
    Physical dimension as a string, e.g., 'meters/second', 'length * second^2'
    """

    description: str = pydantic.Field()
    """
    Detailed description of the symbol
    """

    type: Type = pydantic.Field()
    """
    Mathematical type of the symbol
    """

    validations: typing.Dict[str, ValidationResult] = pydantic.Field()
    """
    List of validation results for the symbol, key is validator's name
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.summarizer_response import SummarizerResponse
from ...core.pydantic_utilities import parse_obj_as
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DocumentClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def summarize(
        self,
        *,
        markdown: str,
        images: typing.Dict[str, str],
        question: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SummarizerResponse:
        """
        Summarize a PIC document

        Parameters
        ----------
        markdown : str

        images : typing.Dict[str, str]

        question : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummarizerResponse
            Successful Response

        Examples
        --------
        from axiomatic import Axiomatic

        client = Axiomatic(
            api_key="YOUR_API_KEY",
        )
        client.pic.document.summarize(
            markdown="markdown",
            images={"key": "value"},
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "pic/circuit/summarize",
            method="POST",
            json={
                "markdown": markdown,
                "images": images,
                "question": question,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SummarizerResponse,
                    parse_obj_as(
                        type_=SummarizerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncDocumentClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def summarize(
        self,
        *,
        markdown: str,
        images: typing.Dict[str, str],
        question: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SummarizerResponse:
        """
        Summarize a PIC document

        Parameters
        ----------
        markdown : str

        images : typing.Dict[str, str]

        question : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SummarizerResponse
            Successful Response

        Examples
        --------
        import asyncio

        from axiomatic import AsyncAxiomatic

        client = AsyncAxiomatic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.pic.document.summarize(
                markdown="markdown",
                images={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "pic/circuit/summarize",
            method="POST",
            json={
                "markdown": markdown,
                "images": images,
                "question": question,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SummarizerResponse,
                    parse_obj_as(
                        type_=SummarizerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .symbol import Symbol
from .validation_result import ValidationResult
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Expression(UniversalBaseModel):
    """
    Represents a mathematical or physical expression with its properties and validation.

    Examples:
        >>> newton_law = Expression(
        ...     name="Newton's Second Law",
        ...     description="The force equals mass times acceleration",
        ...     original_format="F = ma",
        ...     wolfram_expression="F == m*a",
        ...     symbols={
        ...         "F": Symbol(
        ...             name="force",
        ...             wolfram_format="F",
        ...             latex_representation="\vec{F}",
        ...             dimension="mass * length * second^-2",
        ...             description="Net force acting on the object",
        ...             type="vector"
        ...         ),
        ...         "m": Symbol(
        ...             name="mass",
        ...             wolfram_format="m",
        ...             latex_representation="m",
        ...             dimension="mass",
        ...             description="Mass of the object",
        ...             type="scalar"
        ...         )
        ...     },
        ...     narrative_assumptions=[
        ...         "Object is treated as a point mass",
        ...         "Forces are in an inertial reference frame"
        ...     ],
        ...     validation={
        ...         "dimensions": ValidationResult(
        ...             is_valid=True,
        ...             message="Dimensions match on both sides"
        ...         )
        ...     }
        ... )
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the expression
    """

    name: str = pydantic.Field()
    """
    Human-readable name of the Expression (e.g., 'Newton's Law')
    """

    description: str = pydantic.Field()
    """
    Text describing the Expression
    """

    original_format: str = pydantic.Field()
    """
    Original form of the Expression (e.g., LaTeX)
    """

    wolfram_expression: str = pydantic.Field()
    """
    Expression in Wolfram Language (or other symbolic form)
    """

    symbols: typing.Dict[str, Symbol] = pydantic.Field()
    """
    Dictionary mapping symbol names to their Symbol objects
    """

    narrative_assumptions: typing.List[str] = pydantic.Field()
    """
    Narrative text describing assumptions/approximations
    """

    exp_validations: typing.Dict[str, ValidationResult] = pydantic.Field()
    """
    Dictionary mapping validation types to their results
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

StatementType = typing.Union[
    typing.Literal["COST_FUNCTION", "PARAMETER_CONSTRAINT", "UNFORMALIZABLE_STATEMENT"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.named_quantity import NamedQuantity
from ..types.parameter_bound import ParameterBound
from ..types.named_quantity_list import NamedQuantityList
from ..core.request_options import RequestOptions
from ..types.optimizer_result import OptimizerResult
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DigitalTwinClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def optimize(
        self,
        *,
        parameters: typing.Sequence[NamedQuantity],
        parameter_bounds: typing.Sequence[ParameterBound],
        constants: typing.Sequence[NamedQuantity],
        inputs: typing.Sequence[NamedQuantityList],
        targets: typing.Sequence[NamedQuantityList],
        tolerance: typing.Optional[float] = OMIT,
        max_time: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OptimizerResult:
        """
        Fits a digital twin model to input data using numerical optimization.

            Accepts parameter guesses, bounds, inputs, targets, and constants — all with units —
            and returns the optimized parameters that minimize the model's error using a selected cost function.

            Args:
                OptimizerBody: The input data and optimization parameters.
            Returns:
                OptimizerResult: The result of the optimization.

        Parameters
        ----------
        parameters : typing.Sequence[NamedQuantity]

        parameter_bounds : typing.Sequence[ParameterBound]

        constants : typing.Sequence[NamedQuantity]

        inputs : typing.Sequence[NamedQuantityList]

        targets : typing.Sequence[NamedQuantityList]

        tolerance : typing.Optional[float]

        max_time : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OptimizerResult
            Successful Response

        Examples
        --------
        from axiomatic import (
            Axiomatic,
            NamedQuantity,
            NamedQuantityList,
            ParameterBound,
            Quantity,
            QuantityList,
        )

        client = Axiomatic(
            api_key="YOUR_API_KEY",
        )
        client.digital_twin.optimize(
            parameters=[
                NamedQuantity(
                    name="name",
                    value=Quantity(
                        magnitude=1.1,
                        unit="unit",
                    ),
                )
            ],
            parameter_bounds=[
                ParameterBound(
                    name="name",
                    lower=Quantity(
                        magnitude=1.1,
                        unit="unit",
                    ),
                    upper=Quantity(
                        magnitude=1.1,
                        unit="unit",
                    ),
                )
            ],
            constants=[
                NamedQuantity(
                    name="name",
                    value=Quantity(
                        magnitude=1.1,
                        unit="unit",
                    ),
                )
            ],
            inputs=[
                NamedQuantityList(
                    name="name",
                    value=QuantityList(
                        unit="unit",
                        magnitude=[1.1],
                    ),
                )
            ],
            targets=[
                NamedQuantityList(
                    name="name",
                    value=QuantityList(
                        unit="unit",
                        magnitude=[1.1],
                    ),
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "digital-twin/optimize",
            method="POST",
            json={
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=typing.Sequence[NamedQuantity], direction="write"
                ),
                "parameter_bounds": convert_and_respect_annotation_metadata(
                    object_=parameter_bounds, annotation=typing.Sequence[ParameterBound], direction="write"
                ),
                "constants": convert_and_respect_annotation_metadata(
                    object_=constants, annotation=typing.Sequence[NamedQuantity], direction="write"
                ),
                "inputs": convert_and_respect_annotation_metadata(
                    object_=inputs, annotation=typing.Sequence[NamedQuantityList], direction="write"
                ),
                "targets": convert_and_respect_annotation_metadata(
                    object_=targets, annotation=typing.Sequence[NamedQuantityList], direction="write"
                ),
                "tolerance": tolerance,
                "max_time": max_time,
                "model_name": "EOResponseModel",
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    OptimizerResult,
                    parse_obj_as(
                        type_=OptimizerResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncDigitalTwinClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def optimize(
        self,
        *,
        parameters: typing.Sequence[NamedQuantity],
        parameter_bounds: typing.Sequence[ParameterBound],
        constants: typing.Sequence[NamedQuantity],
        inputs: typing.Sequence[NamedQuantityList],
        targets: typing.Sequence[NamedQuantityList],
        tolerance: typing.Optional[float] = OMIT,
        max_time: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OptimizerResult:
        """
        Fits a digital twin model to input data using numerical optimization.

            Accepts parameter guesses, bounds, inputs, targets, and constants — all with units —
            and returns the optimized parameters that minimize the model's error using a selected cost function.

            Args:
                OptimizerBody: The input data and optimization parameters.
            Returns:
                OptimizerResult: The result of the optimization.

        Parameters
        ----------
        parameters : typing.Sequence[NamedQuantity]

        parameter_bounds : typing.Sequence[ParameterBound]

        constants : typing.Sequence[NamedQuantity]

        inputs : typing.Sequence[NamedQuantityList]

        targets : typing.Sequence[NamedQuantityList]

        tolerance : typing.Optional[float]

        max_time : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OptimizerResult
            Successful Response

        Examples
        --------
        import asyncio

        from axiomatic import (
            AsyncAxiomatic,
            NamedQuantity,
            NamedQuantityList,
            ParameterBound,
            Quantity,
            QuantityList,
        )

        client = AsyncAxiomatic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.digital_twin.optimize(
                parameters=[
                    NamedQuantity(
                        name="name",
                        value=Quantity(
                            magnitude=1.1,
                            unit="unit",
                        ),
                    )
                ],
                parameter_bounds=[
                    ParameterBound(
                        name="name",
                        lower=Quantity(
                            magnitude=1.1,
                            unit="unit",
                        ),
                        upper=Quantity(
                            magnitude=1.1,
                            unit="unit",
                        ),
                    )
                ],
                constants=[
                    NamedQuantity(
                        name="name",
                        value=Quantity(
                            magnitude=1.1,
                            unit="unit",
                        ),
                    )
                ],
                inputs=[
                    NamedQuantityList(
                        name="name",
                        value=QuantityList(
                            unit="unit",
                            magnitude=[1.1],
                        ),
                    )
                ],
                targets=[
                    NamedQuantityList(
                        name="name",
                        value=QuantityList(
                            unit="unit",
                            magnitude=[1.1],
                        ),
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "digital-twin/optimize",
            method="POST",
            json={
                "parameters": convert_and_respect_annotation_metadata(
                    object_=parameters, annotation=typing.Sequence[NamedQuantity], direction="write"
                ),
                "parameter_bounds": convert_and_respect_annotation_metadata(
                    object_=parameter_bounds, annotation=typing.Sequence[ParameterBound], direction="write"
                ),
                "constants": convert_and_respect_annotation_metadata(
                    object_=constants, annotation=typing.Sequence[NamedQuantity], direction="write"
                ),
                "inputs": convert_and_respect_annotation_metadata(
                    object_=inputs, annotation=typing.Sequence[NamedQuantityList], direction="write"
                ),
                "targets": convert_and_respect_annotation_metadata(
                    object_=targets, annotation=typing.Sequence[NamedQuantityList], direction="write"
                ),
                "tolerance": tolerance,
                "max_time": max_time,
                "model_name": "EOResponseModel",
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    OptimizerResult,
                    parse_obj_as(
                        type_=OptimizerResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ....types.plot_parser_output import PlotParserOutput
from ....types.plot_parser_output_v2dto import PlotParserOutputV2Dto

PlotPointsResponse = typing.Union[PlotParserOutput, PlotParserOutputV2Dto]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AxesInfo,
    Bounds,
    Bundle,
    BundleSettingsValue,
    ComponentInstanceLayoutInfo,
    Computation,
    ComputationArgumentsValue,
    ContourCv2,
    CostFunction,
    CreateApiKeyResponse,
    CreateApiKeyrequest,
    CreateUserRequest,
    CreateUserResponse,
    ExecuteCodeResponse,
    Expression,
    ExpressionProcessingResponse,
    ExpressionValidationResult,
    ExtractConstantsResponse,
    ExtractTextResponse,
    ExtractedPoint,
    ExtractedPointDto,
    ExtractedPoints,
    ExtractedSeriesDto,
    FindMappingResponse,
    FindUserResponse,
    FormalizeCircuitResponse,
    FormalizeResponse,
    GenerateCodeResponse,
    GenerateComponentCodeResponse,
    GenerateLensCodeResponse,
    GetOptimizableParametersResponse,
    GetSpectrumResponse,
    GetSpectrumResponseSpectrumDbValueItem,
    GetSpectrumResponseSpectrumValueItem,
    HttpValidationError,
    InformalizeStatementResponse,
    LayoutInfo,
    NamedQuantity,
    NamedQuantityList,
    Net,
    Netlist,
    NetlistInputExtraPdkSettingsValue,
    NetlistOutputExtraPdkSettingsValue,
    OptimizationHistory,
    OptimizeConfig,
    OptimizeNetlistResponse,
    OptimizePlacementBodyResponse,
    OptimizerResult,
    Parameter,
    ParameterBound,
    ParameterConstraint,
    ParseMethods,
    ParseResponse,
    ParseStatementResponse,
    PdkType,
    PicInstance,
    PicInstanceInfoValue,
    PicInstanceSettingsValue,
    PicWarning,
    PicZ3Expression,
    Placement,
    PlotInfo,
    PlotInfoXAxisMax,
    PlotInfoXAxisMin,
    PlotInfoXAxisTickValuesItem,
    PlotInfoYAxisMax,
    PlotInfoYAxisMin,
    PlotInfoYAxisTickValuesItem,
    PlotParserOutput,
    PlotParserOutputV2Dto,
    PortInstanceLayoutInfo,
    Quantity,
    QuantityList,
    RefineCodeResponse,
    RefineComponentCodeResponse,
    ScheduleJobResponse,
    SolutionResponse,
    SolutionResponseSolutionValue,
    Statement,
    StatementDictionary,
    StatementType,
    StatementValidation,
    StatementValidationDictionary,
    StatusResponse,
    SummarizerResponse,
    Symbol,
    ToolsListResponse,
    Type,
    UnformalizableStatement,
    UpdateCodeResponse,
    ValidateNetlistResponse,
    ValidateResponse,
    ValidationError,
    ValidationErrorLocItem,
    ValidationRequirement,
    ValidationResult,
    VerifyCircuitCodeResponse,
    VerifyResponse,
)
from .errors import UnprocessableEntityError
from . import code_execution, digital_twin, document, formalization, fso, lean, pic, tools
from .client import AsyncAxiomatic, Axiomatic
from .environment import AxiomaticEnvironment
from .version import __version__

__all__ = [
    "AsyncAxiomatic",
    "AxesInfo",
    "Axiomatic",
    "AxiomaticEnvironment",
    "Bounds",
    "Bundle",
    "BundleSettingsValue",
    "ComponentInstanceLayoutInfo",
    "Computation",
    "ComputationArgumentsValue",
    "ContourCv2",
    "CostFunction",
    "CreateApiKeyResponse",
    "CreateApiKeyrequest",
    "CreateUserRequest",
    "CreateUserResponse",
    "ExecuteCodeResponse",
    "Expression",
    "ExpressionProcessingResponse",
    "ExpressionValidationResult",
    "ExtractConstantsResponse",
    "ExtractTextResponse",
    "ExtractedPoint",
    "ExtractedPointDto",
    "ExtractedPoints",
    "ExtractedSeriesDto",
    "FindMappingResponse",
    "FindUserResponse",
    "FormalizeCircuitResponse",
    "FormalizeResponse",
    "GenerateCodeResponse",
    "GenerateComponentCodeResponse",
    "GenerateLensCodeResponse",
    "GetOptimizableParametersResponse",
    "GetSpectrumResponse",
    "GetSpectrumResponseSpectrumDbValueItem",
    "GetSpectrumResponseSpectrumValueItem",
    "HttpValidationError",
    "InformalizeStatementResponse",
    "LayoutInfo",
    "NamedQuantity",
    "NamedQuantityList",
    "Net",
    "Netlist",
    "NetlistInputExtraPdkSettingsValue",
    "NetlistOutputExtraPdkSettingsValue",
    "OptimizationHistory",
    "OptimizeConfig",
    "OptimizeNetlistResponse",
    "OptimizePlacementBodyResponse",
    "OptimizerResult",
    "Parameter",
    "ParameterBound",
    "ParameterConstraint",
    "ParseMethods",
    "ParseResponse",
    "ParseStatementResponse",
    "PdkType",
    "PicInstance",
    "PicInstanceInfoValue",
    "PicInstanceSettingsValue",
    "PicWarning",
    "PicZ3Expression",
    "Placement",
    "PlotInfo",
    "PlotInfoXAxisMax",
    "PlotInfoXAxisMin",
    "PlotInfoXAxisTickValuesItem",
    "PlotInfoYAxisMax",
    "PlotInfoYAxisMin",
    "PlotInfoYAxisTickValuesItem",
    "PlotParserOutput",
    "PlotParserOutputV2Dto",
    "PortInstanceLayoutInfo",
    "Quantity",
    "QuantityList",
    "RefineCodeResponse",
    "RefineComponentCodeResponse",
    "ScheduleJobResponse",
    "SolutionResponse",
    "SolutionResponseSolutionValue",
    "Statement",
    "StatementDictionary",
    "StatementType",
    "StatementValidation",
    "StatementValidationDictionary",
    "StatusResponse",
    "SummarizerResponse",
    "Symbol",
    "ToolsListResponse",
    "Type",
    "UnformalizableStatement",
    "UnprocessableEntityError",
    "UpdateCodeResponse",
    "ValidateNetlistResponse",
    "ValidateResponse",
    "ValidationError",
    "ValidationErrorLocItem",
    "ValidationRequirement",
    "ValidationResult",
    "VerifyCircuitCodeResponse",
    "VerifyResponse",
    "__version__",
    "code_execution",
    "digital_twin",
    "document",
    "formalization",
    "fso",
    "lean",
    "pic",
    "tools",
]
